/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Application.h"

#include "BaseConfigurationDialog.h"
#include "ColorSelectionFrame.h"
#include "MainWindow.h"

#include <QApplication>

//____________________________________________
Application::Application( CommandLineArguments arguments ) :
    BaseApplication( nullptr, arguments ),
    Counter( "Application" )
{ Debug::Throw( "Application::Application.\n" ); }

//____________________________________________
bool Application::realizeWidget()
{
    Debug::Throw( "Application::realizeWidget.\n" );

    if( !BaseApplication::realizeWidget() ) return false;

    // top widget
    mainWindow_ = new MainWindow;
    mainWindow_->setWindowTitle( applicationName() );

    // configuration
    emit configurationChanged();

    // top widget
    mainWindow_->centerOnDesktop();
    const CommandLineParser parser( commandLineParser( _arguments() ) );
    if( parser.hasOption( "--color" ) )
    {

        mainWindow_->show();
        mainWindow_->colorSelectionFrame().setColor( QColor( parser.option( "--color" ) ) );

    } else if( parser.hasFlag( "--grab" ) ) {

        if( !XmlOptions::get().get<bool>( "AUTO_HIDE" ) ) mainWindow_->show();
        mainWindow_->colorSelectionFrame().colorWidget().click();

    } else mainWindow_->show();

    return true;

}

//____________________________________________
CommandLineParser Application::commandLineParser( CommandLineArguments arguments, bool ignoreWarnings ) const
{
    Debug::Throw( "Application::commandLineParser.\n" );
    CommandLineParser out( BaseApplication::commandLineParser() );
    out.setGroup( CommandLineParser::applicationGroupName );
    out.registerFlag( "--grab", tr( "select a window using mouse pointer before starting the application" ) );
    out.registerOption( CommandLineParser::Tag( "--color", "-c" ), tr( "string" ), tr( "initial color to be displayed" ) );
    if( !arguments.isEmpty() ) out.parse( arguments, ignoreWarnings );
    return out;
}

//____________________________________________
void Application::usage() const
{
    _usage( "xgrabcolor" );
    commandLineParser().usage();
    return;
}

//_______________________________________________
void Application::_configuration()
{

    Debug::Throw( "Application::_configuration.\n" );
    emit saveConfiguration();

    BaseConfigurationDialog dialog( 0, BaseConfigurationDialog::Flag::Default );
    dialog.setWindowTitle( QString( tr( "Configuration - %1" ) ).arg( applicationName() ) );

    connect( &dialog, SIGNAL(configurationChanged()), SIGNAL(configurationChanged()) );
    dialog.centerOnWidget( qApp->activeWindow() );
    dialog.exec();

}

//________________________________________________
bool Application::_processCommand( Server::ServerCommand command )
{

    Debug::Throw( "Application::_processCommand.\n" );
    if( BaseApplication::_processCommand( command ) ) return true;
    if( command.command() == Server::ServerCommand::Raise )
    {
        mainWindow_->uniconify();
        const CommandLineParser parser( commandLineParser( command.arguments() ) );
        if( parser.hasOption( "--color" ) )
        {

            mainWindow_->colorSelectionFrame().setColor( QColor( parser.option( "--color" ) ) );

        } else if( parser.hasFlag( "--grab" ) ) {

            mainWindow_->colorSelectionFrame().colorWidget().click();

        }

        return true;

    } else return false;

}
