/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ColorModel.h"

#include "CustomPixmap.h"
#include "Debug.h"
#include "IconSize.h"
#include "QtUtil.h"

#include <QPainter>

//_______________________________________________
ColorModel::IconCache& ColorModel::_icons()
{
    static IconCache singleton;
    return singleton;
}

//__________________________________________________________________
Qt::ItemFlags ColorModel::flags(const QModelIndex &index) const
{
    if( !contains( index ) ) return Qt::ItemFlags();
    return Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsDragEnabled;
}

//__________________________________________________________________
QVariant ColorModel::data( const QModelIndex& index, int role ) const
{

    // check index
    if( !contains( index ) ) return QVariant();

    // retrieve associated color
    const Base::Color& color( get( index ) );
    if( role == Qt::DisplayRole ) return color.get().name();
    else if( role == Qt::DecorationRole ) return _icon( color );
    else return QVariant();

}

//__________________________________________________________________
QVariant ColorModel::headerData(int section, Qt::Orientation orientation, int role) const
{

    if(
        orientation == Qt::Horizontal &&
        role == Qt::DisplayRole &&
        section >= 0 &&
        section < nColumns )
    { return columnTitles_[section]; }

    // return empty
    return QVariant();

}

//______________________________________________________________________
QMimeData* ColorModel::mimeData(const QModelIndexList &indexes) const
{

    // create mime data
    QMimeData *mimeData = new QMimeData;

    // add plaint-text mime data
    QString buffer;
    bool first( true );
    for( const auto& index:indexes )
    {
        if( !index.isValid() ) continue;
        if( first ) first = false;
        else buffer += "\n";
        buffer += get(index).get().name();
    }

    // set plain text data
    mimeData->setData( "text/plain", qPrintable( buffer ) );

    return mimeData;

}

//____________________________________________________________
void ColorModel::_sort( int column, Qt::SortOrder order )
{
    Debug::Throw() << "ColorModel::sort - column: " << column << " order: " << order << endl;
    std::sort( _get().begin(), _get().end(), SortFTor( column, order ) );
}

//________________________________________________________
bool ColorModel::SortFTor::operator () ( Base::Color first, Base::Color second ) const
{

    if( order_ == Qt::DescendingOrder ) std::swap( first, second );
    return first < second;
}

//______________________________________________________
QIcon ColorModel::_icon( const Base::Color& color )
{

    IconCache::iterator iter( _icons().find( color ) );
    if( iter != _icons().end() ) return iter.value();
    else {

        CustomPixmap pixmap( IconSize::get( IconSize::Large ), CustomPixmap::Flag::All );
        QPainter painter( &pixmap );
        painter.setBrush( color.get() );
        painter.setRenderHints(QPainter::Antialiasing );

        QRect rect( QPoint(0,0), pixmap.size()/pixmap.devicePixelRatio() );
        rect.adjust( 2, 2, -2, -2 );

        painter.setPen( Qt::NoPen );
        if( color.get().alpha() == 255 )
        {
            // full circle for opaque colors
            painter.setBrush( color.get() );
            painter.drawEllipse( rect );

        } else {

            // right side of the circle is transparent
            painter.setBrush( color.get() );
            painter.drawChord( rect, -90*16, 180*16 );

            // left side of circle is opaque
            painter.setBrush( QColor( color.get().rgb() ) );
            painter.drawChord( rect, 90*16, 180*16 );

        }

        // surrounding circle
        painter.setPen( QPalette().color( QPalette::Text ) );
        painter.setBrush( Qt::NoBrush );
        painter.drawEllipse( rect );

        QIcon icon( pixmap );
        _icons()[color] = icon;
        return icon;

    }

}
