/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "MainWindow.h"

#include "Application.h"
#include "BaseStatusBar.h"
#include "ColorList.h"
#include "ColorSelectionFrame.h"
#include "Menu.h"
#include "Singleton.h"
#include "XmlColor.h"
#include "XmlDocument.h"
#include "XmlOptions.h"

#include <QApplication>
#include <QHeaderView>
#include <QLayout>

//_____________________________________________________________
MainWindow::MainWindow( QWidget* parent ):
    BaseMainWindow( parent )
{
    Debug::Throw( "MainWindow::MainWindow.\n" );
    setOptionName( "MAIN_WINDOW" );

    // central widget
    QWidget* main = new QWidget( this );
    QVBoxLayout *mainLayout = new QVBoxLayout;
    main->setLayout( mainLayout );
    setCentralWidget( main );

    // actions
    _installActions();
    auto application( Base::Singleton::get().application<Application>() );
    addAction( &application->closeAction() );

    // color selection frame
    mainLayout->addWidget( selectionFrame_ = new ColorSelectionFrame( this ) );
    addAction( &selectionFrame_->saveAction() );
    connect( &selectionFrame_->saveAction(), SIGNAL(triggered()), SLOT(_add()) );

    // list
    mainLayout->addWidget( list_ = new ColorList(this) );
    list_->list().header()->hide();
    addAction( &list_->removeAction() );

    // connections
    connect( list_->list().selectionModel(), SIGNAL(currentChanged(QModelIndex,QModelIndex)), SLOT(_selectFromList(QModelIndex)) );
    connect( &toggleListAction(), SIGNAL(toggled(bool)), SLOT(_toggleShowList(bool)) );

    // configuration update
    connect( application, SIGNAL(configurationChanged()), SLOT(_updateConfiguration()) );
    connect( application, SIGNAL(saveConfiguration()), SLOT(_write()) );

    // status bar
    BaseStatusBar* statusBar;
    setStatusBar( statusBar = new BaseStatusBar( this ) );
    statusBar->addLabel( 1 );
    statusBar->addClock();

    // menu bar
    setMenuBar( menu_ = new Menu( this ) );

    // set initial focus
    selectionFrame_->colorWidget().setFocus();

    _read();

}

//_____________________________________________________________
void MainWindow::closeEvent( QCloseEvent *event )
{
    Debug::Throw( "MainWindow::closeEvent.\n" );
    _write();
    return BaseMainWindow::closeEvent( event );
}

//_______________________________________
void MainWindow::_updateConfiguration()
{
    Debug::Throw( "MainWindow::_updateConfiguration.\n" );

    // resize
    toggleListAction().setChecked( XmlOptions::get().get<bool>( "SHOW_LIST" ) );

}

//_____________________________________________________________
void MainWindow::_read()
{

    Debug::Throw( "MainWindow::_read\n" );
    File file( XmlOptions::get().raw("RC_FILE") );

    // make sure file is hidden (windows only)
    if( file.localName().startsWith( '.' ) ) file.setHidden();

    // parse the file
    XmlDocument document;
    {
        QFile qfile( file );
        if( !document.setContent( &qfile ) )
        {
            Debug::Throw() << document.error() << endl;
            return;
        }
    }

    // look for relevant element
    QDomNodeList topNodes = document.elementsByTagName( Base::Xml::ColorList );
    if( topNodes.isEmpty() ) return;
    for(QDomNode node = topNodes.at(0).firstChild(); !node.isNull(); node = node.nextSibling() )
    {
        QDomElement element = node.toElement();
        if( element.isNull() ) continue;

        QString tagName( element.tagName() );
        if( tagName == Base::Xml::Color )
        {
            XmlColor color( element );
            if( color.isValid() ) list_->model().add( color );
        }
    }

    // set index selected
    if( list_->model().rowCount() )
    {
        QModelIndex selectedIndex( list_->model().index( 0, 0 ) );
        list_->list().selectionModel()->select( selectedIndex,  QItemSelectionModel::Clear|QItemSelectionModel::Select|QItemSelectionModel::Rows );
        _selectFromList( selectedIndex );
    }

    return;
}

//_____________________________________________________________
void MainWindow::_write()
{

    Debug::Throw( "MainWindow::_write\n" );

    // output file
    QString file( XmlOptions::get().raw("RC_FILE") );

    // create document and read
    XmlDocument document;
    {
        QFile qtfile( file );
        document.setContent( &qtfile );
    }

    // create main element
    QDomElement top = document.createElement( Base::Xml::ColorList );

    // retrieve colors from model
    ColorModel::List colors( list_->model().get() );
    for( const auto& color:colors )
    { top.appendChild( XmlColor( color ).domElement( document.get() ) ); }

    // append top node to document and write
    document.replaceChild( top );
    {
        QFile qfile( file );
        if( qfile.open( QIODevice::WriteOnly ) )
        { qfile.write( document.toByteArray() ); }
    }

}

//_____________________________________________________________
void MainWindow::_add()
{
    Debug::Throw( "MainWindow::_add.\n" );
    list_->add( Base::Color(selectionFrame_->color()) );
}

//____________________________________________________________
void MainWindow::_selectFromList( const QModelIndex &index )
{

    Debug::Throw( "MainWindow::_selectFromList\n" );
    if( index.isValid() )
    { selectionFrame_->setColor( list_->model().get( index ) ); }

}

//_____________________________________________________________
void MainWindow::_toggleShowList( bool state )
{

    Debug::Throw( "MainWindow::_toggleShowList.\n" );
    list_->setVisible( state );
    XmlOptions::get().set<bool>( "SHOW_LIST", state );

}

//_________________________________________________________________________
void MainWindow::_installActions()
{
    Debug::Throw( "MainWindow::_installActions.\n" );

    // toggle list action
    addAction( toggleListAction_ = new QAction( tr( "Show List" ), this ) );
    toggleListAction_->setToolTip( tr( "Show/hide color list" ) );
    toggleListAction_->setCheckable( true );
    toggleListAction_->setChecked( true );

}
